from cml.usecases.usecase import Usecase
from cml.domain import (
    Preprocessor,
    LearnblockIdentifier,
    DataSource,
    FeatureSelector,
    Reconstructor,
    Constructor,
    Deconstructor,
    ProceduralKnowledgeDatabase,
    ConceptualKnowledgeDatabase,
    RelativeFinder
)


class PreprocessUsecase(Usecase):
    def process(self, request):
        preprocesser = Preprocessor(request.pre_settings)
        cleaned_data = preprocesser.clean(request.source_adapter)
        learnblock_identifier = LearnblockIdentifier(request.block_settings,
                                                     request.density_estimator,
                                                     request.relative_extrema)
        data_source = DataSource(cleaned_data,
                                 learnblock_identifier,
                                 settings=request.block_settings)
        return data_source


class FeatureSelectionUsecase(Usecase):
    def process(self, request):
        return FeatureSelector(request.filter_method,
                               request.embedded_method,
                               request.settings)


class ReconstructionUsecase(Usecase):
    def process(self, request):
        return Reconstructor(request.settings,
                             request.ml_models,
                             request.knowledge_domain)


class CreateConstructorUsecase(Usecase):
    def process(self, request):
        constructor = Constructor(request.ml_models, request.settings)
        constructor.construction_type = request.settings.construct_type
        return constructor


class DeconstructionUsecase(Usecase):
    def process(self, request):
        # TODO (dmt): Read an existing database.
        if request.knowledge_type == "conceptual":
            database = ConceptualKnowledgeDatabase(
                request.settings.highest_tier)
        elif request.knowledge_type == "procedural":
            database = ProceduralKnowledgeDatabase(
                request.settings.highest_tier)
        else:
            raise ValueError()
        relative_finder = RelativeFinder()
        database.observer.append(relative_finder)
        return Deconstructor(database,
                             relative_finder,
                             request.source_adapter,
                             request.settings)
